module Main exposing (..)

import Browser
import Html exposing (..)
import Html.Attributes exposing (..)
import Html.Events exposing (..)
import Character exposing (Character)

main : Program () Model Msg
main = 
    Browser.document
        { init = init
        , view = view
        , update = update
        , subscriptions = subscriptions
        }

init : () -> ( Model, Cmd Msg )
init _ =
    (   { character = Character.new
        , modal = Nothing
        , modalValue = ""
        }
    , Cmd.none
    )

type alias Model = 
    { character : Character
    , modal : Maybe ModalType
    , modalValue : String
    }

type Msg 
    = OpenModal ModalType
    | CloseModal
    | ChangeModalValue String
    | SaveModalChange ModalType

type ModalType
    = Name

update : Msg -> Model -> ( Model, Cmd Msg )
update msg model = 
    case msg of
        OpenModal modalType -> 
            (   { model 
                | modal = ( Just modalType ) 
                , modalValue = modalValue modalType model.character
                }
            , Cmd.none
            )
        CloseModal ->
            ( { model | modal = Nothing }
            , Cmd.none
            )
        ChangeModalValue value ->
            ( { model | modalValue = value }
            , Cmd.none
            )
        SaveModalChange modalType ->
            case modalType of
                Name ->
                    (   { model 
                        | character = Character.changeName model.character model.modalValue 
                        , modal = Nothing
                        }
                    , Cmd.none
                    )

view : Model -> Browser.Document Msg
view model =
    { title = "Character Sheet"
    , body = 
        [ div [] 
            [ button 
                [ class "button"
                , onClick ( OpenModal Name ) 
                ] [ text model.character.name ]
            ]
        , viewModal model
        ]
    }

viewModal : Model -> Html Msg
viewModal model =
    case model.modal of
        Nothing -> div [ class "modal" ] []
        Just modalType ->
            div [ class "modal is-active" ]
                [ div 
                    [ class "modal-background" 
                    , onClick CloseModal
                    ] []
                , div [ class "modal-card" ] 
                    [ header [ class "modal-card-head" ]
                        [ p [ class "modal-card-title" ] [ text ( "Change " ++ ( modalTitle modalType ) ) ]
                        ]
                    , section [ class "modal-card-body" ] 
                        [ p [] 
                            [ input 
                                [ class "input" 
                                , type_ "text"
                                , placeholder ( "Enter New " ++ ( modalTitle modalType ) )
                                , value model.modalValue
                                , onInput ( \x -> ChangeModalValue x )
                                ] []
                            ]
                        ]
                    , footer [ class "modal-card-foot" ] 
                        [ div [ class "buttons" ] 
                            [ button 
                                [ class "button is-success"
                                , onClick ( SaveModalChange modalType )
                                ] [ text "Save Changes" ]
                            , button 
                                [ class "button"
                                , onClick CloseModal
                                ] [ text "Cancel" ]
                            ]
                        ]
                    ]
                ]

modalTitle : ModalType -> String
modalTitle modalType =
    case modalType of
        Name -> "Character Name"

modalValue : ModalType -> Character -> String
modalValue modalType character =
    case modalType of
        Name -> character.name

subscriptions : Model -> Sub Msg
subscriptions _ =
  Sub.none

ariaLabel : String -> Attribute msg
ariaLabel value =
    Html.Attributes.attribute "aria-label" value